require 'autocommit/detector'

describe "Detecting File Changes" do
  def create_detector
    `sync`  # prevent setup from interfering with rb-inotify
    @detector = AutoCommit::Detector.new TmpDir do |event|
      path = event.absolute_name
      path.slice! TmpDir
      @changed << [path, event.flags]
    end
    # two tricks to let the Notifier really run
    Thread.pass
    sleep 0.001
  end

  before :each do
    @changed = []
  end

  after :each do
    @detector.stop
  end

  describe "Creation" do
    before :each do
      create_detector
    end

    it "of One file" do
      create "bla.txt"
      check ["bla.txt", [:create]]
    end

    it "of Two files" do
      create "one", "two"
      check ["one", [:create]], ["two", [:create]]
    end

    it "of a Directory" do
      mkdir "sub"
      check ["sub", [:create, :isdir]]
    end
  end

  it "Deletion of a file" do
    create "bla.txt"
    create_detector
    remove "bla.txt"
    check ["bla.txt", [:delete]]
  end

  it "Rename of a file" do
    create "bla.txt"
    create_detector
    move("bla.txt").to "real.rb"
    check ["bla.txt", [:moved_from, :move]], ["real.rb", [:moved_to, :move]]
  end

  it "Modification of a file" do
    create "bla.txt"
    create_detector
    modify "bla.txt"
    check ["bla.txt", [:modify]]
  end

  describe "in a Sub-directory" do
    before :each do
      mkdir "sub"
    end

    it "Creation of a file" do
      create_detector
      create "sub/bla.txt"
      check ["sub/bla.txt", [:create]]
    end

    it "Creation of a directory" do
      create_detector
      mkdir "sub/dir"
      check ["sub/dir", [:create, :isdir]]
    end

    it "Modification of a file" do
      create "sub/bla.txt"
      modify "sub/bla.txt" # just for fun
      create_detector
      modify "sub/bla.txt"
      check ["sub/bla.txt", [:modify]]
    end

    it "Removal of a file" do
      create "sub/bla.txt"
      create_detector
      remove "sub/bla.txt"
      check ["sub/bla.txt", [:delete]]
    end

    it "Move a file out of the directory" do
      create "sub/bla.txt"
      create_detector
      move("sub/bla.txt").to "real.rb"
      check ["sub/bla.txt", [:moved_from, :move]], ["real.rb", [:moved_to, :move]]
    end

    it "Move a file into the directory" do
      create "bla.txt"
      create_detector
      move("bla.txt").to "sub/real.rb"
      check ["bla.txt", [:moved_from, :move]], ["sub/real.rb", [:moved_to, :move]]
    end
  end

  describe "in a New Sub-directory" do
    before :each do
      create_detector
      mkdir "sub"
    end

    # This is a race, by the nature of inotify
    it "Creation of a file" do
      sleep 0.025  # give rb-inotify time to create a watcher on the dir
      create "sub/bla.txt"
      check ["sub", [:create, :isdir]], ["sub/bla.txt", [:create]]
    end
  end

  def check *events
    10.times do
      break if @changed == events
      sleep 0.05
    end
    expect(@changed).to eq events
  end
end
